package com.maenle.bump

import androidx.test.ext.junit.runners.AndroidJUnit4
import androidx.test.platform.app.InstrumentationRegistry
import com.maenle.bump.util.LocalData
import com.maenle.bump.util.MessageProcessor
import org.junit.Assert
import org.junit.Test
import org.junit.runner.RunWith

@RunWith(AndroidJUnit4::class)
class MessageProcessorTest {

    @Test
    fun decryptMessage() {
        val code = "dydoes-unknowledgeable-indiscretion-househusbands-pot-walloper-indiscretion-discophorous-transcriptions-dydoes-poodle-faker-transcriptions-budlike"
        // val messageRaw = "M1dEAxKZ5HUHCJoRkgGOvAABhqCAAAAAAGG2eKTSlKXWLDQx5B_wssZsNwsanzQID2UyUm4KKuKYKgfwH5MG2N-qzt6K4mg3pfZmWPaiDB9PiqlX236k6zo9Yvvq"
        val messageRaw = "M1dEAxKZ5HUHCJoRkgGOvAABhqCAAAAAAGG8afPPk380EzwcbGzNoTr_I4y6YT8hnUYcToinlgsVkaUx5K-JicdS5epZenOX4u8vVhhMvR0ebeWm_mgp6LZvTw8S"
        val processor = MessageProcessor(code)
        val decrypted = processor.decrypt(messageRaw)
        Assert.assertEquals(decrypted, "hello")
    }

    @Test
    fun encryptAndDecryptMessage() {
        val code = "dydoes-unknowledgeable-indiscretion-househusbands-pot-walloper-indiscretion-discophorous-transcriptions-dydoes-poodle-faker-transcriptions-budlike"
        val message = MessageProcessor(code)

        val test = getRandomString(32)
        val encrypted = message.encrypt(test)

        val processor = MessageProcessor(code)
        val decrypted = processor.decrypt(encrypted)
        Assert.assertEquals(test, decrypted)
    }

    @Test
    fun setCode() {
        val test = getRandomString(512)
        val appContext = InstrumentationRegistry.getInstrumentation().targetContext
        val local = LocalData(appContext)
        local.code = test
        Assert.assertEquals(local.code, test)
    }

    companion object {
        @JvmStatic
        fun decryptMessage(code: String, messageData: String): String {
            val message = MessageProcessor(code)
            val data = message.decrypt(messageData)
            return data
        }
    }
}
