package com.maenle.bump


import android.app.NotificationManager
import android.util.Log
import androidx.core.app.NotificationCompat
import androidx.core.content.ContextCompat
import com.maenle.bump.util.sendNotification
import com.google.firebase.messaging.FirebaseMessagingService
import com.google.firebase.messaging.RemoteMessage
import com.maenle.bump.util.BumpProcessor

class MyFirebaseMessagingService : FirebaseMessagingService() {

    /**
     * Called when message is received.
     *
     * @param remoteMessage Object representing the message received from Firebase Cloud Messaging.
     */
    override fun onMessageReceived(remoteMessage: RemoteMessage) {
        // Not getting messages here? See why this may be: https://goo.gl/39bRNJ
        // Log.d(TAG, "From: ${remoteMessage.from}")

        // Check if message contains a data payload.
        var body: String? = null;
        var header: String? = null;
        remoteMessage.data.let {
            val bump = BumpProcessor.getInstance(applicationContext)
            Log.d(TAG, "Message data payload: " + remoteMessage.data)
            val a = remoteMessage.data.keys
            val value = remoteMessage.data["data"]
            value?.let {
                body = bump.decryptMessage(it)
            }
        }

        remoteMessage.notification?.let {
            header = it.body
        }

        sendNotification(header, body)

    }

    // log registration token
    // [START on_new_token]
    /**
     * Called if InstanceID token is updated. This may occur if the security of
     * the previous token had been compromised. Note that this is called when the InstanceID token
     * is initially generated so this is where you would retrieve the token.
     */
    override fun onNewToken(token: String) {

        // If you want to send messages to this application instance or
        // manage this apps subscriptions on the server side, send the
        // Instance ID token to your app server.
        sendRegistrationToServer()
    }
    // [END on_new_token]


    /**
     * Persist token to third-party servers.
     *
     * @param token The new token.
     */
    private fun sendRegistrationToServer() {
        val bump = BumpProcessor.getInstance(baseContext)
        bump.setFirebaseToken(baseContext)
    }

    private fun sendNotification(messageHeader: String?, messageBody: String?) {

        val notificationManager = ContextCompat.getSystemService(applicationContext, NotificationManager::class.java) as NotificationManager
        notificationManager.sendNotification(messageHeader, messageBody, applicationContext)

    }

    companion object {
        private const val TAG = "MyFirebaseMsgService"
    }
}