package com.maenle.bump.ui

import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Intent
import android.content.res.Configuration
import android.graphics.Bitmap
import android.graphics.Color
import android.net.Uri
import android.os.Bundle
import android.view.*
import androidmads.library.qrgenearator.QRGContents
import androidmads.library.qrgenearator.QRGEncoder
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import com.google.firebase.messaging.FirebaseMessaging
import com.google.zxing.WriterException
import com.maenle.bump.R
import com.maenle.bump.databinding.FragmentFirstBinding
import com.maenle.bump.util.*
import com.maenle.bump.util.Display
import org.json.JSONArray
import org.json.JSONObject


class FirstFragment : Fragment() {

    private var _binding: FragmentFirstBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    private lateinit var local: LocalData
    private var message: MessageProcessor? = null

    private lateinit var bump: BumpProcessor
    override fun onCreateView(
            inflater: LayoutInflater, container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View {

        local = LocalData(requireContext())
        message = local.code?.let { MessageProcessor(it) }
        _binding = FragmentFirstBinding.inflate(inflater, container, false)
        bump = BumpProcessor(requireContext())


        createChannel(
            getString(R.string.bump_notification_channel_id),
            getString(R.string.bump_notification_channel_name)
        )

        subscribeTopic()

        bump.callback = { log -> updateLogCallback(log) }
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        binding.buttonScan.setOnClickListener {
            findNavController().navigate(R.id.action_FirstFragment_to_CameraFragment)
        }
        binding.buttonView.setOnClickListener {
            if(binding.textviewEncryption.visibility == View.GONE){
                binding.textviewEncryption.visibility = View.VISIBLE
                binding.idIVQrcode.visibility = View.VISIBLE
                binding.buttonView.text = getString(R.string.hide_sender)
                message?.let { binding.textviewScan.text = it.sender.replace("-", "-\u200b")}
            } else {
                binding.textviewEncryption.visibility = View.GONE
                binding.idIVQrcode.visibility = View.GONE
                binding.buttonView.text = getString(R.string.view_sender)
                message?.let { binding.textviewScan.text = it.sender.replace("-", "-\u200b")}
            }
        }
        message?.let { binding.textviewScan.text = it.sender.replace("-", "-\u200b")}
        message?.let { binding.textviewEncryption.text = "-" + it.password.replace("-", "-\u200b")}
        message?.let {binding.idIVQrcode.setImageBitmap(generateQrCode(it.code))}
    }

    private fun createChannel(channelId: String, channelName: String) {
        val notificationChannel = NotificationChannel(
            channelId,
            channelName,
            NotificationManager.IMPORTANCE_HIGH
        )
            .apply {
                setShowBadge(false)
            }

        notificationChannel.enableLights(true)
        notificationChannel.lightColor = Color.RED
        notificationChannel.enableVibration(true)
        notificationChannel.description = getString(R.string.bump_notification_channel_id)

        val notificationManager = requireActivity().getSystemService(
            NotificationManager::class.java
        )
        notificationManager.createNotificationChannel(notificationChannel)
    }

    private fun generateQrCode(code: String): Bitmap? {

        val nightModeFlags = requireContext().resources.configuration.uiMode and
                Configuration.UI_MODE_NIGHT_MASK

        val display = Display(requireActivity())
        // generating dimension from width and height.
        var dimen = if (display.width < display.height)  display.width else display.height
        dimen = dimen * 3 / 4

        val qrgEncoder = QRGEncoder(code, null, QRGContents.Type.TEXT, dimen)
        var bitmap = Bitmap.createBitmap(display.width, display.height, Bitmap.Config.ARGB_8888)
        try {
            bitmap = qrgEncoder.encodeAsBitmap()
        } catch (e: WriterException) {
            // Log.d("Tag", e.toString())
        }

        if(nightModeFlags == Configuration.UI_MODE_NIGHT_YES) {
            bitmap = ChangeColor(bitmap)
        }
        return bitmap
    }

    private fun ChangeColor(scrBitmap: Bitmap): Bitmap {
        // make an empty bitmap the same size as scrBitmap
        val newBitmap = Bitmap.createBitmap(scrBitmap.width, scrBitmap.height, Bitmap.Config.ARGB_8888)

        for (i in 0 until newBitmap.width) {
            for (j in 0 until newBitmap.height) {
                // get the pixel from the scrBitmap image
                val pixel = scrBitmap.getPixel(i, j)
                if(pixel == Color.WHITE) {
                    newBitmap.setPixel(i, j, Color.BLACK)
                } else {
                    newBitmap.setPixel(i, j, Color.WHITE)
                }
            }
        }
        return newBitmap
    }

    private fun updateLogCallback(log: JSONArray) {
        val messages: MutableList<MessageItem> = mutableListOf()
        val texts: MutableList<String> = mutableListOf()
        binding.nrMessages.text = log.length().toString()
        for(l in 0 until log.length()) {
            val m = log[l] as JSONObject
            messages.add(MessageItem(m.get("title") as String,
                                     m.get("data") as String))
            texts.add(m.get("title") as String)
        }
        val adapter = NotificationAdapter(requireContext(), messages)
        binding.notificationList.adapter = adapter
        binding.notificationList.setOnItemClickListener { adapterView, view, i, l ->
            run {
                if(messages[i].header.startsWith("http")) {
                    val browserIntent =
                        Intent(Intent.ACTION_VIEW, Uri.parse(messages[i].header))
                    startActivity(browserIntent)
                }
                if(messages[i].message == null) {
                    messages[i].message = bump.decryptMessage(messages[i].encode)
                }

                val body:String
                if(messages[i].showTitle) {
                    body = messages[i].message!!
                    messages[i].showTitle = false
                } else {
                    body = messages[i].title
                    messages[i].showTitle = true
                }
                if(body == "") {
                    messages[i].header = "-"
                } else {
                    messages[i].header = body
                }
                adapter.notifyDataSetChanged()
                // Log.d(TAG, bump.decryptMessage(messages[i].body))
            }
        }
    }

    private fun subscribeTopic() {
        FirebaseMessaging.getInstance().subscribeToTopic(TOPIC)
            .addOnCompleteListener { task ->
                var curmessage = getString(R.string.message_subscribed)
                if (!task.isSuccessful) {
                    curmessage = getString(R.string.message_subscribe_failed)
                }
                // Log.d(TAG, curmessage.toString())
            }
        // [END subscribe_topics]
    }

    companion object {
        private val TAG = MainActivity::class.java.simpleName
        private const val TOPIC = "Bump"
        fun newInstance() = FirstFragment()
    }


    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}

class MessageItem(public val title: String, var encode: String) {
    public var showTitle = true
    public var header: String = title
    public var message:String? = null
}