package com.maenle.bump.util

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.google.firebase.messaging.FirebaseMessaging
import org.json.JSONArray
import org.json.JSONObject
import java.io.File

class BumpProcessor constructor(context: Context) {

    private val rest: RestSingleton = RestSingleton.getInstance(context)
    private val local = LocalData(context)
    private var log: JSONArray
    private val messenger = local.code?.let { MessageProcessor(it) }

    var callback: ((JSONArray)->Unit)? = null

    init {
        log = getLog(context)
        startUpdateHandler(context)
    }

    companion object {
        @Volatile
        private var INSTANCE: BumpProcessor? = null
        fun getInstance(context: Context) =
            INSTANCE ?: synchronized(this) {
                INSTANCE ?: BumpProcessor(context).also {
                    INSTANCE = it
                }
            }
        fun clearLog(context: Context) {
            val logFile = File(context.filesDir, ".bump_log")
            logFile.writeText("")
        }

    }

    fun decryptMessage(message: String): String {
        return messenger!!.decrypt(message)
    }

    private fun updateLog(context: Context, list: JSONArray) {
        log = getLog(context)
        for(i in 0 until list.length()) {
            var exists = false
            for(j in 0 until log.length()) {
                if (list[i].toString() == log[j].toString()) {
                    exists = true
                    break
                }
            }
            if(!exists) {
                log.put(list[i])
                addToLog(context, list[i] as JSONObject)
            }
        }

        callback?.invoke(log)
    }

    private fun getSecret(context: Context):String? {
        val local = LocalData(context)
        return local.code
    }

    private fun startUpdateHandler(context: Context) {
        val delay: Long = 60_000 //milliseconds
        val handler = Handler(Looper.getMainLooper())

        updateFromServer(context)
        handler.postDelayed(object : Runnable {
            override fun run() {
                updateFromServer(context)
                handler.postDelayed(this, delay)
            }
        }, delay)
    }

    fun updateFromServer(context: Context) {
        LocalData(context).code?.let {
            val messenger = MessageProcessor(it)
            rest.list(messenger.sender) { list -> updateLog(context, list) }
        }
    }

    private fun addToLog(context: Context, line: JSONObject) {
        val logFile = File(context.filesDir, ".bump_log")
        logFile.appendText(line.toString() + "\n")
    }

    fun addSecret(context: Context, newCode: String): Boolean {
        return if(!checkCodeValidity(newCode)) {
            false
        } else {
            val local = LocalData(context)
            local.code = newCode

            setFirebaseToken(context)
            true
        }
    }

    private fun checkCodeValidity(newCode: String): Boolean {
        val validChars = newCode.filter{ it in 'A'..'Z' || it in 'a'..'z' || it == '-'}.length
        return newCode.length == validChars && newCode.length >= 32
    }

    private fun getLog(context: Context): JSONArray {
        val logFile = File(context.filesDir, ".bump_log")
        val log = JSONArray()
        !logFile.exists().let {
            logFile.createNewFile()
        }
        for(line in logFile.readLines()) {
            log.put(JSONObject(line))
        }
        return log
    }

    fun setFirebaseToken(context: Context) {
        val tokenTask = FirebaseMessaging.getInstance().token
        val secret = LocalData(context).code
        val messenger = secret?.let { MessageProcessor(it) }
        tokenTask.addOnSuccessListener {  token -> ( messenger?.let { rest.firebase(messenger.sender, token) { result -> Log.d("result", result.toString())} } ) }
    }

}