package com.maenle.bump.util

import android.app.Activity
import android.graphics.Insets
import android.graphics.Point
import android.os.Build
import android.util.DisplayMetrics
import android.view.WindowInsets
import androidx.camera.core.AspectRatio
import com.maenle.bump.ui.CameraFragment
import com.maenle.bump.ui.MainActivity
import kotlin.math.abs
import kotlin.math.max
import kotlin.math.min

class Display(activity: Activity) {
    private var pwidth: Int = 0
    private var pheight: Int = 0
    private var pAspectRatio: Int = 0

    init {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            val windowMetrics = activity.windowManager.currentWindowMetrics
            val insets = windowMetrics.windowInsets
                .getInsetsIgnoringVisibility(WindowInsets.Type.systemBars())
            pwidth = windowMetrics.bounds.width()
            pheight = windowMetrics.bounds.height()
            pAspectRatio = width - insets.left - insets.right
        } else {
            val displayMetrics = DisplayMetrics()
            activity.windowManager.defaultDisplay.getMetrics(displayMetrics)
            pwidth = displayMetrics.widthPixels
            pheight = displayMetrics.heightPixels
            pAspectRatio = aspectRatio(displayMetrics.widthPixels, displayMetrics.heightPixels)
        }
    }

    val width: Int
        get() {
            return pwidth
    }
    val height: Int
        get() {
            return pheight
        }

    val aspectRatio: Int
        get() {
            return pAspectRatio
        }

    /**
     *  [androidx.camera.core.ImageAnalysis], [androidx.camera.core.Preview] requires enum value of
     *  [androidx.camera.core.AspectRatio]. Currently it has values of 4:3 & 16:9.
     *
     *  Detecting the most suitable ratio for dimensions provided in @params by counting absolute
     *  of preview ratio to one of the provided values.
     *
     *  @param width - preview width
     *  @param height - preview height
     *  @return suitable aspect ratio
     */
    private fun aspectRatio(width: Int, height: Int): Int {
        val previewRatio = max(width, height).toDouble() / min(width, height)
        if (abs(previewRatio - RATIO_4_3_VALUE) <= abs(previewRatio - RATIO_16_9_VALUE)) {
            return AspectRatio.RATIO_4_3
        }
        return AspectRatio.RATIO_16_9
    }

    companion object {


        private val TAG = MainActivity::class.java.simpleName
        private const val PERMISSION_CAMERA_REQUEST = 1

        private const val RATIO_4_3_VALUE = 4.0 / 3.0
        private const val RATIO_16_9_VALUE = 16.0 / 9.0
    }
}