package com.maenle.bump.util

import android.app.NotificationManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import androidx.core.app.NotificationCompat
import com.maenle.bump.ui.MainActivity
import com.maenle.bump.R




// Notification ID.
private const val NOTIFICATION_ID = 0

fun NotificationManager.sendNotification(messageHeader: String?, messageBody: String?, applicationContext: Context) {
    var title = applicationContext.getString(R.string.notification_title)
    var body = ""

    if(messageBody != null) {
        if(messageHeader != null) {
            title = messageHeader
        }
        body = messageBody
    } else if(messageHeader != null)  {
        body = messageHeader
    }

    sendParsedNotification(title, body, applicationContext)
}

fun NotificationManager.sendParsedNotification(messageHeader: String, messageBody: String, applicationContext: Context) {
    // Create the content intent for the notification, which launches
    // this activity
    val contentIntent = Intent(applicationContext, MainActivity::class.java)
    val contentPendingIntent = PendingIntent.getActivity(
        applicationContext,
        NOTIFICATION_ID,
        contentIntent,
        PendingIntent.FLAG_UPDATE_CURRENT
    )

    val eggImage = BitmapFactory.decodeResource(
        applicationContext.resources,
        R.drawable.ic_stat_name
    )

    val intent = Intent(applicationContext, this.javaClass)
    val action =
        PendingIntent.getBroadcast(applicationContext, 0, intent, 0)
    // Build the notification
    val builder = NotificationCompat.Builder(
        applicationContext,
        applicationContext.getString(R.string.bump_notification_channel_id)
    )

        .setSmallIcon(R.drawable.ic_stat_name)
        .setContentTitle(messageHeader)
        .setContentText(messageBody)

        // set content intent
        .setContentIntent(contentPendingIntent)
        .setAutoCancel(true)

        // add style to builder
        .setLargeIcon(eggImage)

        // set priority
        .setPriority(NotificationCompat.PRIORITY_HIGH)
        .addAction(0, "Action", action)

    // notify
    notify(NOTIFICATION_ID, builder.build())
}

fun NotificationManager.cancelNotifications() {
    cancelAll()
}