package com.maenle.bump.util

import android.content.Context
import android.util.Log
import com.android.volley.Request
import com.android.volley.RequestQueue
import com.android.volley.toolbox.*
import com.maenle.bump.ui.MainActivity
import org.json.JSONArray

import org.json.JSONObject

class RestSingleton constructor(context: Context){

    init {
        MessageProcessor
    }
    companion object {
        @Volatile
        private var INSTANCE: RestSingleton? = null
        fun getInstance(context: Context) =
            INSTANCE ?: synchronized(this) {
                INSTANCE ?: RestSingleton(context).also {
                    INSTANCE = it
                }
            }

        private val TAG = MainActivity::class.java.simpleName
        private const val URL = "https://bump.maenle.net/api/"
    }

    fun list(sender: String, callback: (JSONArray) -> Unit){
        val url = URL + "list/"
        val data = JSONObject()
        data.put("sender", sender)
        data.put("minutes", (60*24).toString())
        val jsonRequest = JsonObjectRequest(Request.Method.POST, url,
            data,
            { response ->
                run {
                    callback(response.get("messages") as JSONArray)
                }
            } ,
            {callback(JSONArray())})

        requestQueue.add(jsonRequest)
    }

    fun peek(sender: String, callback: (JSONObject) -> Unit){
        val url = URL + "peek/"
        val data = JSONObject()
        data.put("sender", sender)
        val jsonRequest = JsonObjectRequest(Request.Method.POST, url,
            data,
            { response -> callback(JSONObject(response.toString())) },
            {callback(JSONObject())})

        requestQueue.add(jsonRequest)
    }

    fun push(sender: String, title: String, encoded: String, function: () -> Unit){
        val url = URL + "push/"
        val data = JSONObject()
        data.put("sender", sender)
        data.put("title", title)
        data.put("data", encoded)
        val stringRequest = JsonObjectRequest(Request.Method.POST, url,
            data,
            {_ -> function()},
            {Log.d(TAG, "none")}
        )

        requestQueue.add(stringRequest)
    }

    fun pop(sender: String,callback: (JSONObject) -> Unit){
        val url = URL + "pop/"
        val data = JSONObject()
        data.put("sender", sender)
        val stringRequest = JsonObjectRequest(Request.Method.POST, url,
            data,
            { response -> callback(JSONObject(response.toString())) },
            {callback(JSONObject())})

        requestQueue.add(stringRequest)
    }

    fun firebase(sender: String, firebase_token: String, callback: (JSONObject) -> Unit) {
        val url = URL + "firebase/"
        val data = JSONObject()
        data.put("sender", sender)
        data.put("token", firebase_token)

        val stringRequest = JsonObjectRequest(Request.Method.POST, url,
            data,
            { response -> callback(JSONObject(response.toString())) },
            {callback(JSONObject())})

        requestQueue.add(stringRequest)

    }

    private val requestQueue: RequestQueue by lazy {
        // applicationContext is key, it keeps you from leaking the
        // Activity or BroadcastReceiver if someone passes one in.
        Volley.newRequestQueue(context.applicationContext)
    }
}